@@ -167,41 +167,70 @@ void do_map_drive(void)
     cfg_unset(cf_other,"map-drive");
 }
 
-
-/*
-   Return the derived "table=", if possible
-*/
-char *boot_table (char *boot)
+/* 
+ *  Derive the name of the MBR from the partition name
+ *  e.g.
+ *   /dev/scsi/host2/bus0/target1/lun0/part2	=> disc
+ *   /dev/sd/c0b0t0u0p7				=> c0b0t0u0
+ *   /dev/sda11					=> sda
+ *
+ * If table==0, do no check for primary partition; if table==1, check
+ * that we started from a primary (1-4) partition.
+ *
+ * A NULL return indicates an error
+ *
+ */
+ 
+char *boot_mbr(const char *boot, int table)
 {
-    int i;
-    char *npart;
-    char *part = NULL;
+    char *part, *npart, *endptr;
+    int i, j, k;
     
     npart = stralloc(boot);
-    i = strlen(boot);
-  /* handle primary partitions on IDE & SCSI disks */
-    if (i==9 && 
-	(strncmp(boot,"/dev/hd",7)==0 || strncmp(boot,"/dev/sd",7)==0) &&
-	    boot[8]>='1' && boot[8]<='4') {
-		part = strcpy(npart, boot);
-		part[8] = 0;
+    part = strrchr(npart, '/');
+    if (!part++) die ("No '/' in partition/device name.");
+    
+    i = strlen(part);
+    endptr = part + i - 1;
+    
+   /* j is the count of digits at the end of the name */ 
+    j = 0;
+    while (isdigit(*endptr)) { j++; --endptr; }
+    if (j==0 && !table) die ("Not a partition name; no digits at the end.");
+    
+    k = !table || (j==1 && endptr[1]>='1' && endptr[1]<='4');
+    
+   /* test for devfs  partNN */
+    if (strncmp(part, "part", 4)==0) {
+    	strcpy(part, "disc");
+    } 
+   /* test for ..NpNN */
+    else if (*endptr=='p' && isdigit(endptr[-1])) {
+        *endptr = 0;  /* truncate the pNN part */
     }
-  /* handle disks under devfs */
-    else if (strncmp("/part", boot+i-6, 5)==0 &&
-    	    boot[i-1]>='1' && boot[i-1]<='4') {
-		part = strcpy(npart, boot);
-		strcpy(npart-5+i, "disc");
+   /* test for old /dev/hda3 or /dev/sda11 */
+    else if (endptr[-1]=='d' && endptr[-3]=='/' &&
+    		(endptr[-2]=='h' || endptr[-2]=='s')
+#if 0
+		&& strstr(npart, "/dev/")
+#endif
+	    ) {
+        endptr[1] = 0;  /* truncate the NN part */
     }
-  /* handle /dev/rd/c0d0p1 (Mylex & Compaq controllers) */
-    else if (isdigit(boot[i-1]) && boot[i-2] == 'p' && isdigit(boot[i-3]) &&
-    	    boot[i-1]>='1' && boot[i-1]<='4') {
-    	        part = strcpy(npart, boot);
-    	        part[i-2] = 0;
+    else 
+	k = 0;
+
+    if (verbose>=3) {
+        printf("Name: %s  yields MBR: %s  (with%s primary partition check)\n",
+           boot, k ? npart : "(NULL)", table ? "" : "out");
     }
-    return part;
+
+    if (k) return npart;
+    else return NULL;
 }
 
 
+
 #define PART(s,n) (((struct partition *) (s)[0].par_c.ptable)[(n)])
 
 
@@ -226,28 +255,20 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
     	    else letter = 0;
     	}
     }
-#if 1
-    if (!part) part = boot_table(boot);
+    if (!part) part = boot_mbr(boot, 1);
     /* part may still be NULL */
-#else
-    if (!part) {   /*  "/dev/hdXN and /dev/sdXN have default partition tables -- JRC */
-        if (strlen(boot)==9 && 
-       	  (strncmp(boot,"/dev/hd",7)==0 || strncmp(boot,"/dev/sd",7)==0) &&
-       	  boot[8]>='1' && boot[8]<='4') {
-	    part = strcpy(npart, boot);
-	    part[8] = 0;
-	}
-    }
-#endif
+
     if (verbose > 0)
 	printf("Boot other: %s%s%s, loader %s\n",boot,part ? ", on " : "",part
 	  ? part : "",loader);
 
+#ifdef LCF_AUTOAUTO
     if (!cfg_get_flag(cf_other, "change")) {
     	autoauto = 1;	/* flag that change rules may be automatically inserted */
         do_cr_auto();
         autoauto = 0;
     }
+#endif    
 
     if (cfg_get_flag(cf_other,"unsafe")) {
 	(void) geo_open_boot(&geo,boot);
