@@ -1,6 +1,13 @@
 /* common.c  -  Common data structures and functions. */
+/*
+Copyright 1992-1998 Werner Almesberger.
+Copyright 1999-2001 John Coffman.
+All rights reserved.
 
-/* Copyright 1992-1998 Werner Almesberger. See file COPYING for details. */
+Licensed under the terms contained in the file 'COPYING' in the 
+source directory.
+
+*/
 
 
 #include <unistd.h>
@@ -13,16 +20,19 @@
 #include "lilo.h"
 
 
-int verbose = 0,test = 0,compact = 0,linear = 0,nowarn = 0, lba32 = 0,
-					autoauto = 0;
+int verbose = 0,test = 0,compact = 0,linear = 0, raid_flags = 0, zflag = 0,
+      nowarn = 0, lba32 = 0, autoauto = 0, passw = 0, geometric = 0;
 
 unsigned short drv_map[DRVMAP_SIZE+1]; /* fixup maps ... */
 int curr_drv_map;
 unsigned long prt_map[PRTMAP_SIZE+1];
 int curr_prt_map;
+#if 0
+unsigned long crc_polynomial[MAX_PW_CRC] = { 
+	CRC_POLY1, CRC_POLY2, CRC_POLY3, CRC_POLY4 };
+#endif	
 
-
-volatile void pdie(char *msg)
+/*volatile*/ void pdie(char *msg)
 {
     fflush(stdout);
     perror(msg);
@@ -30,16 +40,16 @@ volatile void pdie(char *msg)
 }
 
 
-volatile void die(char *fmt,...)
+/*volatile*/ void die(char *fmt,...)
 {
     va_list ap;
 
     fflush(stdout);
-    fprintf(stderr,"Fatal: ");       /* JRC */
+    fprintf(errstd,"Fatal: ");       /* JRC */
     va_start(ap,fmt);
-    vfprintf(stderr,fmt,ap);
+    vfprintf(errstd,fmt,ap);
     va_end(ap);
-    fputc('\n',stderr);
+    fputc('\n',errstd);
     exit(1);
 }
 
@@ -49,6 +59,8 @@ void *alloc(int size)
     void *this;
 
     if ((this = malloc(size)) == NULL) pdie("Out of memory");
+    memset(this, 0, size);	/* insure consistency */
+
     return this;
 }
 
@@ -106,26 +118,14 @@ void check_version(BOOT_SECTOR *sect,int stage)
 	die("%s has a pre-21 LILO signature",name(stage));
     if (strncmp(sect->par_1.signature,"LILO",4))
 	die("%s doesn't have a valid LILO signature",name(stage));
-    if (sect->par_1.stage != stage)
+    if ((sect->par_1.stage&0xFF) != stage)
 	die("%s has an invalid stage code (%d)",name(stage),sect->par_1.stage);
 
     bs_major = sect->par_1.version & 255;
     bs_minor = sect->par_1.version >> 8;
-#if VERSION_MINOR<999
     if (sect->par_1.version != VERSION)
-#else
-    if (bs_major != VERSION_MAJOR)
-#endif
 	die("%s is version %d.%d. Expecting version %d.%d.",name(stage),
 	    bs_major,bs_minor, VERSION_MAJOR,VERSION_MINOR);
-#if 0
-    if (lba32) {
-        if (strncmp(sect->sector+3, "lba", 3)) {
-	    if (verbose > 0) printf("sector+3 -> %s\n", sect->sector+3);
-	    die("%s is not the LBA32 version.", name(stage));
-	}
-    }
-#endif
 }
 
 
@@ -133,3 +133,24 @@ int stat_equal(struct stat *a,struct stat *b)
 {
     return a->st_dev == b->st_dev && a->st_ino == b->st_ino;
 }
+
+/* calculate a CRC-32 polynomial */
+
+unsigned long crc32 (unsigned char *cp, int nsize, unsigned long polynomial)
+{
+   unsigned long poly, crc;
+   int i;
+   unsigned char ch;
+
+   crc = 0xFFFFFFFFUL;
+   while (nsize--) {
+      ch = *cp++;
+      for (i=0; i<8; i++) {
+         if ( ( (crc>>31) ^ (ch>>(7-i)) ) & 1) poly = polynomial;
+         else poly = 0UL;
+         crc = (crc<<1) ^ poly;
+      }
+   }
+   return ~crc;
+}
+
