@@ -1,19 +1,28 @@
 /* geometry.c  -  Device and file geometry computation */
+/*
+Copyright 1992-1998 Werner Almesberger.
+Copyright 1999-2001 John Coffman.
+All rights reserved.
 
-/* Copyright 1992-1998 Werner Almesberger. See file COPYING for details. */
+Licensed under the terms contained in the file 'COPYING' in the 
+source directory.
+
+*/
 /* Patched for linux-2.4.0 - Glibc-2.2 by Sergey Ostrovsky 11/16/2000 */
 
 #include <unistd.h>
-#include <sys/types.h>
-#include <sys/statfs.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <errno.h>
+#include <sys/types.h>
+
+#ifdef LCF_REISERFS
+#include <sys/statfs.h>
 
 #ifdef	_SYS_STATFS_H
 #define	_I386_STATFS_H	/* two versions of statfs is not good ... */
 #endif
-
+#endif
 #include <linux/fs.h>
 #include <linux/hdreg.h>
 #include <linux/fd.h>
@@ -26,6 +35,7 @@
 #include "geometry.h"
 #include "cfg.h"
 #include "md-int.h"
+#include "probe.h"
 
 #ifdef LCF_REISERFS
 #ifndef REISERFS_SUPER_MAGIC
@@ -39,6 +49,20 @@
 #endif
 #endif
 
+#ifdef LCF_LVM
+struct lv_bmap {
+    __u32 lv_block;
+    __u16 lv_dev;
+};
+
+#ifndef LV_BMAP
+#define LV_BMAP				_IOWR(0xfe, 0x30, 1)
+#endif
+#ifndef LVM_GET_IOP_VERSION
+#define LVM_GET_IOP_VERSION		_IOR(0xfe, 0x98, 1)
+#endif
+#endif
+
 #ifndef HDIO_GETGEO
 #define HDIO_GETGEO HDIO_REQ
 #endif
@@ -103,7 +127,7 @@ void do_partition(void)
     entry->start = -1;
     partition = cfg_get_strg(cf_partitions,"partition");
     if (stat(partition,&st) < 0) die("stat %s: %s",partition,strerror(errno));
-    if (!S_ISBLK(st.st_mode) || ((st.st_rdev ^ disktab->device) & ~PART_MASK))
+    if (!S_ISBLK(st.st_mode) || ((st.st_rdev ^ disktab->device) & D_MASK(st.st_rdev)))
 	die("%s is not a valid partition device",partition);
     entry->device = st.st_rdev;
     cfg_init(cf_partition);
@@ -121,22 +145,28 @@ void do_partition(void)
 
 int has_partitions(dev_t dev)
 {
+    int major = MAJOR(dev);
+    
     if (
-      MAJOR(dev) == MAJOR_HD || MAJOR(dev) == MAJOR_IDE2 ||
-      MAJOR(dev) == MAJOR_IDE3 || MAJOR(dev) == MAJOR_IDE4 ||
-      MAJOR(dev) == MAJOR_IDE5 || MAJOR(dev) == MAJOR_IDE6 ||
-      MAJOR(dev) == MAJOR_XT || MAJOR(dev) == MAJOR_SD ||
-      MAJOR(dev) == MAJOR_SD2 || MAJOR(dev) == MAJOR_SD3 ||
-      MAJOR(dev) == MAJOR_SD4 || MAJOR(dev) == MAJOR_ESDI ||
-      MAJOR(dev) == MAJOR_AMI_HYP ||
-      (MAJOR(dev) >= MAJOR_IDE7 && MAJOR(dev) <= MAJOR_IDE10) ||
-      (MAJOR(dev) >= MAJOR_I2O && MAJOR(dev) <= MAJOR_I2O+7) ||
-      (MAJOR(dev) >= COMPAQ_SMART2_MAJOR && MAJOR(dev) <= COMPAQ_SMART2_MAJOR+7) ||
-      (MAJOR(dev) >= COMPAQ_CISS_MAJOR && MAJOR(dev) <= COMPAQ_CISS_MAJOR+7)
+      major == MAJOR_HD || major == MAJOR_IDE2 ||
+      major == MAJOR_IDE3 || major == MAJOR_IDE4 ||
+      major == MAJOR_IDE5 || major == MAJOR_IDE6 ||
+      (major >= MAJOR_IDE7 && major <= MAJOR_IDE10) ||
+      major == MAJOR_XT || major == MAJOR_ESDI
+      	) return 0xFFC0;
+      
+    if (
+      major == MAJOR_SD || (major >= MAJOR_SD2 && major <= MAJOR_SD8) ||
+      major == MAJOR_AMI_HYP || major == MAJOR_HPT370 ||
+      major == MAJOR_FL || major == MAJOR_NFTL ||
+      (major >= MAJOR_I2O && major <= MAJOR_I2O+7) ||
+      (major >= MAJOR_SMART2 && major <= MAJOR_SMART2+7) ||
+      (major >= MAJOR_CISS && major <= MAJOR_CISS+7)
         ) return 0xFFF0;	/* 4 bit partition mask */
 
-    if (MAJOR(dev) >= MAJOR_DAC960 && MAJOR(dev) <= MAJOR_DAC960+7)
-          return 0xFFF8;	/* 3 bit partition mask */
+    if ( major == MAJOR_IBM_iSER ||
+      (major >= MAJOR_DAC960 && major <= MAJOR_DAC960+7)
+        )  return 0xFFF8;	/* 3 bit partition mask */
 
     return 0;
 }
@@ -151,7 +181,7 @@ void do_disk(void)
     entry = alloc_t(DT_ENTRY);
     disk = cfg_get_strg(cf_options,"disk");
     if (stat(disk,&st) < 0) die("stat %s: %s",disk,strerror(errno));
-    if (!S_ISBLK(st.st_mode) || ((MINOR(st.st_rdev) & PART_MASK) &&
+    if (!S_ISBLK(st.st_mode) || ((MINOR(st.st_rdev) & P_MASK(st.st_rdev)) &&
       has_partitions(st.st_rdev)))
 	die("RSN: %s is not a whole disk device",disk);
     entry->device = st.st_rdev;
@@ -278,7 +308,44 @@ static int last_dev(int major,int increment)
 }
 
 
-void geo_query_dev(GEOMETRY *geo,int device,int all)
+#ifdef LCF_LVM
+void lvm_bmap(struct lv_bmap *lbm)
+{
+    DEVICE dev;
+    static int lvmfd = -1;
+    static dev_t last_dev = 0;
+
+    if (lbm->lv_dev != last_dev) {
+	char lvm_char[] = "/dev/lvm";
+	unsigned short iop;
+
+	if (lvmfd != -1)
+	    close(lvmfd);
+
+	if ((lvmfd = open(lvm_char, lbm->lv_dev, O_RDONLY)) < 0)
+	    die("can't open LVM char device %s\n", lvm_char);
+
+	if (ioctl(lvmfd, LVM_GET_IOP_VERSION, &iop) < 0)
+	    die("LVM_GET_IOP_VERSION failed on %s\n", lvm_char);
+
+	if (iop < 10)
+	    die("LVM IOP %d not supported for booting\n", iop);
+	close(lvmfd);
+
+	lvmfd = dev_open(&dev, lbm->lv_dev, O_RDONLY);
+	if (lvmfd < 0)
+	    die("can't open LVM block device %#x\n", lbm->lv_dev);
+	last_dev = lbm->lv_dev;
+    }
+    if (ioctl(lvmfd, LV_BMAP, lbm) < 0) {
+	perror(__FUNCTION__);
+	pdie("LV_BMAP error or ioctl unsupported, can't have image in LVM.\n");
+    }
+}
+#endif
+
+
+static void geo_query_dev(GEOMETRY *geo,int device,int all)
 {
     DEVICE dev;
     int fd,get_all;
@@ -329,16 +396,6 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	    /* fall through */
 	case MAJOR_XT:
 	    /* fall through */
-	case MAJOR_I2O:
-	case MAJOR_I2O+1:
-	case MAJOR_I2O+2:
-	case MAJOR_I2O+3:
-	case MAJOR_I2O+4:
-	case MAJOR_I2O+5:
-	case MAJOR_I2O+6:
-	case MAJOR_I2O+7:
-	    geo->device = 0x80+(MINOR(device) >> 6)+(MAJOR(device) == MAJOR_HD ?
-	      0 : last_dev(MAJOR_HD,64));
 	    if (ioctl(fd,HDIO_GETGEO,&hdprm) < 0)
 		die("geo_query_dev HDIO_GETGEO (dev 0x%04x): %s",device,
 		  strerror(errno));
@@ -346,12 +403,18 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	    geo->cylinders = hdprm.cylinders;
 	    geo->sectors = hdprm.sectors;
 	    geo->start = hdprm.start;
+	    if ((geo->device = bios_device(geo, device)) < 0)
+		geo->device = 0x80 + (MINOR(device) >> 6) +
+		    (MAJOR(device) == MAJOR_HD ? 0 : last_dev(MAJOR_HD,64));
 	    break;
 	case MAJOR_SD:
 	case MAJOR_SD2:
 	case MAJOR_SD3:
 	case MAJOR_SD4:
-	    geo->device = 0x80+last_dev(MAJOR_HD,64)+(MINOR(device) >> 4);
+	case MAJOR_SD5:
+	case MAJOR_SD6:
+	case MAJOR_SD7:
+	case MAJOR_SD8:
 	    if (ioctl(fd,HDIO_GETGEO,&hdprm) < 0)
 		die("geo_query_dev HDIO_GETGEO (dev 0x%04x): %s",device,
 		  strerror(errno));
@@ -362,6 +425,8 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	    geo->cylinders = hdprm.cylinders;
 	    geo->sectors = hdprm.sectors;
 	    geo->start = hdprm.start;
+	    if ((geo->device = bios_device(geo, device)) < 0)
+		geo->device = 0x80 + last_dev(MAJOR_HD,64) + (MINOR(device) >> 4);
 	    break;
 	case MAJOR_DAC960:
 	case MAJOR_DAC960+1:
@@ -371,36 +436,48 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	case MAJOR_DAC960+5:
 	case MAJOR_DAC960+6:
 	case MAJOR_DAC960+7:
-	    geo->device = 0x80+last_dev(MAJOR_HD,64)+(MINOR(device) >> 3);
+	case MAJOR_IBM_iSER:
 	    if (ioctl(fd,HDIO_GETGEO,&hdprm) < 0)
 		die("geo_query_dev HDIO_GETGEO (dev 0x%04x): %s",device,
 		  strerror(errno));
 	    if (all && !hdprm.sectors)
-		die("HDIO_REQ not supported for your DAC960 controller. "
+		die("HDIO_REQ not supported for your DAC960/IBM controller. "
 		  "Please use a DISK section");
 	    geo->heads = hdprm.heads;
 	    geo->cylinders = hdprm.cylinders;
 	    geo->sectors = hdprm.sectors;
 	    geo->start = hdprm.start;
+	    if ((geo->device = bios_device(geo, device)) < 0)
+		geo->device = 0x80 + last_dev(MAJOR_HD,64) + (MINOR(device) >> 3);
 	    break;
 	case MAJOR_AMI_HYP:
-	case COMPAQ_SMART2_MAJOR+0:
-	case COMPAQ_SMART2_MAJOR+1:
-	case COMPAQ_SMART2_MAJOR+2:
-	case COMPAQ_SMART2_MAJOR+3:
-	case COMPAQ_SMART2_MAJOR+4:
-	case COMPAQ_SMART2_MAJOR+5:
-	case COMPAQ_SMART2_MAJOR+6:
-	case COMPAQ_SMART2_MAJOR+7:
-	case COMPAQ_CISS_MAJOR+0:
-	case COMPAQ_CISS_MAJOR+1:
-	case COMPAQ_CISS_MAJOR+2:
-	case COMPAQ_CISS_MAJOR+3:
-	case COMPAQ_CISS_MAJOR+4:
-	case COMPAQ_CISS_MAJOR+5:
-	case COMPAQ_CISS_MAJOR+6:
-	case COMPAQ_CISS_MAJOR+7:
-	    geo->device = 0x80+last_dev(MAJOR_HD,64)+(MINOR(device) >> 4);
+	case MAJOR_HPT370:
+	case MAJOR_FL:
+	case MAJOR_NFTL:
+	case MAJOR_SMART2+0:
+	case MAJOR_SMART2+1:
+	case MAJOR_SMART2+2:
+	case MAJOR_SMART2+3:
+	case MAJOR_SMART2+4:
+	case MAJOR_SMART2+5:
+	case MAJOR_SMART2+6:
+	case MAJOR_SMART2+7:
+	case MAJOR_CISS+0:
+	case MAJOR_CISS+1:
+	case MAJOR_CISS+2:
+	case MAJOR_CISS+3:
+	case MAJOR_CISS+4:
+	case MAJOR_CISS+5:
+	case MAJOR_CISS+6:
+	case MAJOR_CISS+7:
+	case MAJOR_I2O:
+	case MAJOR_I2O+1:
+	case MAJOR_I2O+2:
+	case MAJOR_I2O+3:
+	case MAJOR_I2O+4:
+	case MAJOR_I2O+5:
+	case MAJOR_I2O+6:
+	case MAJOR_I2O+7:
 	    if (ioctl(fd,HDIO_GETGEO,&hdprm) < 0)
 		die("geo_query_dev HDIO_GETGEO (dev 0x%04x): %s",device,
 		  strerror(errno));
@@ -411,6 +488,8 @@ void geo_query_dev(GEOMETRY *geo,int device,int all)
 	    geo->cylinders = hdprm.cylinders;
 	    geo->sectors = hdprm.sectors;
 	    geo->start = hdprm.start;
+	    if ((geo->device = bios_device(geo, device)) < 0)
+		geo->device = 0x80 + last_dev(MAJOR_HD,64) + (MINOR(device) >> 4);
 	    break;
 
 	default:
@@ -428,7 +507,7 @@ int is_first(int device)
 	if (walk->device == device) break;
     if (!walk && !old_disktab)
 	for (walk = disktab; walk; walk = walk->next)
-	    if (walk->device == (device & ~PART_MASK)) break;
+	    if (walk->device == (device & D_MASK(device))) break;
     if (walk && !walk->heads)
 	die("Device 0x%04X: Configured as inaccessible.\n",device);
     if (walk && walk->bios != -1) return !(walk->bios & 0x7f);
@@ -446,16 +525,62 @@ int is_first(int device)
 	case MAJOR_IDE5:
 	    /* fall through */
 	case MAJOR_IDE6:
+	case MAJOR_IDE7:
+	case MAJOR_IDE8:
+	case MAJOR_IDE9:
+	case MAJOR_IDE10:
 	    /* fall through */
 	case MAJOR_ESDI:
 	    /* fall through */
 	case MAJOR_XT:
 	    /* fall through */
-	case MAJOR_I2O:
 	    return MINOR(device) >> 6 ? 0 : !last_dev(MAJOR_HD,64);
 	case MAJOR_SD:
+	case MAJOR_SD2:
+	case MAJOR_SD3:
+	case MAJOR_SD4:
+	case MAJOR_SD5:
+	case MAJOR_SD6:
+	case MAJOR_SD7:
+	case MAJOR_SD8:
+	case MAJOR_AMI_HYP:
+	case MAJOR_HPT370:
+	case MAJOR_FL:
+	case MAJOR_NFTL:
+	case MAJOR_SMART2+0:
+	case MAJOR_SMART2+1:
+	case MAJOR_SMART2+2:
+	case MAJOR_SMART2+3:
+	case MAJOR_SMART2+4:
+	case MAJOR_SMART2+5:
+	case MAJOR_SMART2+6:
+	case MAJOR_SMART2+7:
+	case MAJOR_CISS+0:
+	case MAJOR_CISS+1:
+	case MAJOR_CISS+2:
+	case MAJOR_CISS+3:
+	case MAJOR_CISS+4:
+	case MAJOR_CISS+5:
+	case MAJOR_CISS+6:
+	case MAJOR_CISS+7:
+	case MAJOR_I2O:
+	case MAJOR_I2O+1:
+	case MAJOR_I2O+2:
+	case MAJOR_I2O+3:
+	case MAJOR_I2O+4:
+	case MAJOR_I2O+5:
+	case MAJOR_I2O+6:
+	case MAJOR_I2O+7:
 	    return MINOR(device) >> 4 ? 0 : !last_dev(MAJOR_HD,64);
 	case MAJOR_DAC960:
+	case MAJOR_DAC960+1:
+	case MAJOR_DAC960+2:
+	case MAJOR_DAC960+3:
+	case MAJOR_DAC960+4:
+	case MAJOR_DAC960+5:
+	case MAJOR_DAC960+6:
+	case MAJOR_DAC960+7:
+	case MAJOR_IBM_iSER:
 	    return MINOR(device) >> 3 ? 0 : !last_dev(MAJOR_HD,64);
 	default:
 	    return 1; /* user knows what (s)he's doing ... I hope */
@@ -468,11 +593,42 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
     DT_ENTRY *walk;
     int inherited,keep_cyls;
 
+#ifdef LCF_LVM
+    /*
+     * Find underlying device (PV) for LVM.  It is OK if the underlying PV is
+     * really an MD RAID1 device, because the geometry of the RAID1 device is
+     * exactly the same as the underlying disk, so FIBMAP and LV_BMAP should
+     * return the correct block numbers regardless of MD.
+     *
+     * We do a quick test to see if the LVM LV_BMAP ioctl is working correctly.
+     * It should map the two blocks with the same difference as they were input,
+     * with a constant offset from their original block numbers.  If this is not
+     * the case then LV_BMAP is not working correctly (some widely distributed
+     * kernels did not have working LV_BMAP support, some just oops here).
+     */
+    if (MAJOR(device) == MAJOR_LVM)
+    {
+	struct lv_bmap lbmA, lbmB;
+#define DIFF 255
+
+	lbmA.lv_dev = lbmB.lv_dev = device;
+	lbmA.lv_block = 0;
+	lbmB.lv_block = DIFF;
+
+	lvm_bmap(&lbmA);
+	lvm_bmap(&lbmB);
+	if (lbmB.lv_block - lbmA.lv_block != DIFF)
+	    die("This version of LVM does not support boot LVs");
+	device = geo->base_dev = lbmA.lv_dev;
+    }
+#endif
+    /* Find underlying device for MD RAID */
     if (MAJOR(device) == MD_MAJOR) {
         char mdxxx[16];
 	int md_fd, pass;
 	md_array_info_t md_array_info;
 	md_disk_info_t md_disk_info;
+	int raid_limit;
 
 	sprintf(mdxxx, "/dev/md%d", MINOR(device));
 	if ((md_fd=open(mdxxx,O_NOACCESS)) < 0)
@@ -487,7 +643,8 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
 	    die("Raid versions < 0.90 are not supported");
 	if (md_array_info.level != 1)
 	    die("Only RAID1 devices are supported for boot images");
-	for (pass = 0; pass < md_array_info.nr_disks; ++pass) {
+	raid_limit = md_array_info.raid_disks + md_array_info.spare_disks;
+   	for (pass = 0; pass < raid_limit; pass++) {
 	    md_disk_info.number = pass;
 	    if (ioctl(md_fd,GET_DISK_INFO,&md_disk_info) < 0)
 	        die("GET_DISK_INFO: %s", mdxxx);
@@ -503,16 +660,16 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
     inherited = !walk && !old_disktab;
     if (inherited)
 	for (walk = disktab; walk; walk = walk->next)
-	    if (walk->device == (device & ~PART_MASK)) break;
+	    if (walk->device == (device & D_MASK(device))) break;
     if (walk && !walk->heads)
 	die("Device 0x%04X: Configured as inaccessible.\n",device);
     keep_cyls = !walk || walk->bios == -1 || walk->heads == -1 ||
       walk->sectors == -1 || inherited || walk->start == -1;
     if (keep_cyls) {
 	geo_query_dev(geo,device,all);
-	if (!nowarn && (geo->device & 0x7f) >= BIOS_MAX_DEVS &&
+	if (!nowarn && (geo->device & 0x7f) >= bios_max_devs() &&
 	  user_device == -1 && (!walk || walk->bios == -1))
-	    fprintf(stderr,"Warning: BIOS drive 0x%02x may not be accessible\n",
+	    fprintf(errstd,"Warning: BIOS drive 0x%02x may not be accessible\n",
 	      geo->device);
     }
     if (walk) {
@@ -542,7 +699,7 @@ void geo_get(GEOMETRY *geo,int device,int user_device,int all)
     if (!lba32 &&
       (geo->start+geo->sectors-1)/geo->heads/geo->sectors >= BIOS_MAX_CYLS &&
       !nowarn) {
-	fprintf(stderr,"Warning: device 0x%04x exceeds %d cylinder limit.\n"
+	fprintf(errstd,"Warning: device 0x%04x exceeds %d cylinder limit.\n"
         "   Use of the 'lba32' option may help on newer (EDD BIOS) systems.\n",
 	  device,BIOS_MAX_CYLS);
     }
@@ -564,7 +721,7 @@ int geo_open(GEOMETRY *geo,char *name,int flags)
 
     if ((here = strrchr(name,':')) == NULL) user_dev = -1;
     else {
-	fprintf(stderr,":BIOS syntax is no longer supported. Please use a "
+	fprintf(errstd,":BIOS syntax is no longer supported. Please use a "
 	  "DISK section\n");
 	*here++ = 0;
 	user_dev = to_number(here);
@@ -574,8 +731,9 @@ int geo_open(GEOMETRY *geo,char *name,int flags)
     if (fstat(geo->fd,&st) < 0) die("fstat %s: %s",name,strerror(errno));
     if (!S_ISREG(st.st_mode) && !S_ISBLK(st.st_mode))
 	die("%s: neither a reg. file nor a block dev.",name);
-    geo_get(geo,S_ISREG(st.st_mode) ? st.st_dev : st.st_rdev,user_dev,1);
-    geo->file = S_ISREG(st.st_mode);
+    geo->dev = S_ISREG(st.st_mode) ? st.st_dev : st.st_rdev;
+    geo_get(geo, geo->dev, user_dev, 1);
+    geo->file = S_ISREG(st.st_mode) ? st.st_dev : 0;
     geo->boot = 0;
 #ifndef FIGETBSZ
     geo->spb = 2;
@@ -583,7 +741,7 @@ int geo_open(GEOMETRY *geo,char *name,int flags)
     if (!geo->file) geo->spb = 2;
     else {
 	if (ioctl(geo->fd,FIGETBSZ,&block_size) < 0) {
-	    fprintf(stderr,"FIGETBSZ %s: %s\n",name,strerror(errno));
+	    fprintf(errstd,"FIGETBSZ %s: %s\n",name,strerror(errno));
 	    geo->spb = 2;
 	}
 	else {
@@ -600,16 +758,15 @@ int geo_open(GEOMETRY *geo,char *name,int flags)
 int geo_open_boot(GEOMETRY *geo,char *name)
 {
     struct stat st;
-    dev_t dev;
 
     if (stat(name,&st) < 0) die("stat %s: %s",name,strerror(errno));
     if (!S_ISREG(st.st_mode) && !S_ISBLK(st.st_mode))
 	die("%s: neither a reg. file nor a block dev.",name);
-    dev = S_ISREG(st.st_mode) ? st.st_dev : st.st_rdev;
-    if (MAJOR(dev) == MAJOR_FD) geo->fd = 0;
+    geo->dev = S_ISREG(st.st_mode) ? st.st_dev : st.st_rdev;
+    if (MAJOR(geo->dev) == MAJOR_FD) geo->fd = 0;
     else if ((geo->fd = open(name,O_NOACCESS)) < 0)
 	    die("open %s: %s",name,strerror(errno));
-    geo_get(geo,dev,-1,0);
+    geo_get(geo, geo->dev, -1, 0);
     geo->file = S_ISREG(st.st_mode);
     geo->boot = 1;
     geo->spb = 1;
@@ -631,6 +788,7 @@ void geo_close(GEOMETRY *geo)
 int geo_comp_addr(GEOMETRY *geo,int offset,SECTOR_ADDR *addr)
 {
     int block,sector;
+    static int warnings = 0;
 
     if (linear && lba32)
        die("'linear' and 'lba32' (-l and -L) are mutually exclusive.");
@@ -653,18 +811,43 @@ int geo_comp_addr(GEOMETRY *geo,int offset,SECTOR_ADDR *addr)
 	    return 0;
 	}
     }
+#ifdef LCF_LVM
+    if (MAJOR(geo->dev) == MAJOR_LVM) {
+	struct lv_bmap lbm;
+
+	lbm.lv_dev = geo->dev;
+	lbm.lv_block = block;
+
+	lvm_bmap(&lbm);
+	if (lbm.lv_dev != geo->base_dev)
+	    die("LVM boot LV cannot be on multiple PVs\n");
+	block = lbm.lv_block;
+    }
+#endif
     sector = block*geo->spb+((offset/SECTOR_SIZE) % geo->spb);
     sector += geo->start;
  /*   Always use CHS addressing on floppies:     JRC   */
     if ((geo->device & 0x80) && (linear || lba32)) {
-        addr->device = geo->device | (linear ? LINEAR_FLAG : (LBA32_FLAG|LBA32_NOCOUNT));
+        addr->device = geo->device | (linear ? LINEAR_FLAG : (LBA32_FLAG|LBA32_NOCOUNT))
+       		| (do_md_install && geo->file==boot_dev_nr ? RAID_REL_FLAG : 0);
         addr->num_sect = linear ? 1 : (sector >> 24);
 	addr->sector = sector & 0xff;
 	addr->track = (sector >> 8) & 0xff;
 	addr->head = sector >> 16;
-        if (linear && sector>>24)
-	    die("sector %d too large for linear mode"
-					" (try 'lba32' instead)", sector);
+	if (linear) {
+	    int cyl = sector;
+	    if (geo->sectors>0 && geo->heads>0) {
+	    	cyl /= geo->sectors;
+	    	cyl /= geo->heads;
+	    	if (cyl >= BIOS_MAX_CYLS && !nowarn && warnings++ < 8) {
+		    fprintf(errstd,
+			"Warning:  LINEAR may generate cylinder# above 1023 at boot-time.\n");
+	    	}
+	    }
+            if (sector/(63*255) >= BIOS_MAX_CYLS)
+		die("Sector address %d too large for LINEAR"
+					" (try LBA32 instead).", sector);
+	}
 	if (verbose > 4)
 	    printf("fd %d: offset %d -> dev 0x%02x, %s %d\n",
 		         geo->fd, offset, addr->device,
