@@ -16,6 +16,7 @@ typedef struct {
     int spb; /* sectors per block */
     int fd,file;
     int boot; /* non-zero after geo_open_boot */
+    dev_t dev, base_dev; /* real device if remapping (LVM, etc) */
 } GEOMETRY;
 
 typedef struct _dt_entry {
@@ -77,7 +78,7 @@ int geo_find(GEOMETRY *geo,SECTOR_ADDR addr);
 /* lseeks in the file associated with GEO for the sector at address ADDR.
    Returns a non-zero integer on success, zero on failure. */
 
-void geo_query_dev(GEOMETRY *geo,int device,int all);
+/* static void geo_query_dev(GEOMETRY *geo,int device,int all);  */
 
 /* opens the specified device and gets the geometry information.  That
    information is then stored in *geo */
