@@ -1,16 +1,28 @@
 /* lilo.h  -  LILO constants */
+/*
+Copyright 1992-1998 Werner Almesberger.
+Copyright 1999-2001 John Coffman.
+All rights reserved.
 
-/* Copyright 1992-1998 Werner Almesberger. See file COPYING for details. */
+Licensed under the terms contained in the file 'COPYING' in the 
+source directory.
+
+*/
 
 
 /* This file is also included by the boot loader assembler code. Put
    everything that isn't legal C syntax or isn't swallowed by the
    preprocessor into #ifdef LILO_ASM ... #endif */
 
-
 #ifndef LILO_H
 #define LILO_H
 
+/* This is the stuff to check the configuration:
+ */
+#if defined(LCF_READONLY) && defined(LCF_REWRITE_TABLE)
+#error "Incompatible Makefile options: READONLY and REWRITE_TABLE"
+#endif
+
 /*
  * Starting with 2.1.something, Linux kernels put VGA constants and segment
  * definitions into asm/boot.h instead of linux/config.h
@@ -33,9 +45,7 @@
 #define MAJOR_FD	2 /* floppy disks */
 #define MAJOR_HD	3 /* IDE-type hard disks */
 #define MAJOR_SD	8 /* SCSI disks 0-15 */
-#define MAJOR_SD2	65 /* SCSI disks 16-31 */
-#define MAJOR_SD3	66 /* SCSI disks 32-47 */
-#define MAJOR_SD4	67 /* SCSI disks 48-63 */
+#define MAJOR_MD	9 /* multi-disk RAID sets */
 #define MAJOR_XT	13 /* XT-type hard disks */
 #define MAJOR_IDE2	22 /* IDE on second interface */
 #define MAJOR_IDE3	33 /* IDE on third interface */
@@ -44,28 +54,63 @@
 #define MAJOR_DAC960	48 /* First Mylex DAC960 PCI RAID controller */
 #define MAJOR_IDE5	56 /* IDE on fifth interface */
 #define MAJOR_IDE6	57 /* IDE on sixth interface */
+#define MAJOR_LVM	58 /* Logical Volume Manager block device */
+#define MAJOR_FL	62 /* M-Systems Disk-On-Chip 2000 ***experimental*** */
+#define MAJOR_SD2	65 /* SCSI disks 16-31 */
+#define MAJOR_SD3	66 /* SCSI disks 32-47 */
+#define MAJOR_SD4	67 /* SCSI disks 48-63 */
+#define MAJOR_SD5	68 /* SCSI disks 64-79 */
+#define MAJOR_SD6	69 /* SCSI disks 80-95 */
+#define MAJOR_SD7	70 /* SCSI disks 96-111 */
+#define MAJOR_SD8	71 /* SCSI disks 112-127 */
+#define MAJOR_SMART2	72 /* First Compaq Smart/2 Major 72-79 */
+#define MAJOR_I2O	80  /* First I2O block device 80-87 */
 #define MAJOR_IDE7	88 /* IDE on seventh interface */
 #define MAJOR_IDE8	89 /* IDE on eighth interface */
 #define MAJOR_IDE9	90 /* IDE on ninth interface */
 #define MAJOR_IDE10	91 /* IDE on tenth interface */
-#define MAJOR_I2O	80  /* First I2O block device 80-87 */
-#define COMPAQ_SMART2_MAJOR	72 /* First Smart/2 Major 72-79 */
-#define COMPAQ_CISS_MAJOR	104 /* First CCISS Major 104-111 */
+#define MAJOR_NFTL	93 /* NAND Flash Translation Layer (Disk-On-Chip) */
 #define MAJOR_AMI_HYP	101 /* AMI Hyper Disk RAID controller */
+#define MAJOR_CISS	104 /* First CCISS Major 104-111 */
+#define MAJOR_IBM_iSER	112 /* IBM iSeries virtual disk */
+#define MAJOR_HPT370	114 /* HPT370 controller */
 
-#define MAX_IMAGES      ((SECTOR_SIZE*2-2)/sizeof(IMAGE_DESCR))
-			  /* maximum number of images */
+#define MAX_RAID	30	/* max number of RAID disks in a set */
+#define MAX_TOKEN	255 /* max device Token length */
 #define MAX_IMAGE_NAME	15 /* maximum name length (w/o terminating NUL) */
-#define MAX_PW		15 /* maximum password length */
 
-#define MAX_SECONDARY 12
+#ifdef LCF_PASS160
+#define SHS_PASSWORDS		/* use this one if SHS passwords are in use */
+#define MAX_PW_CRC	5  	/* max # of longwords in password digest */
+#define PW_FILE_SUFFIX ".shs"  	/* suffix for the file that saves password digest */
+#else
+#define CRC_PASSWORDS		/* use this one if CRC passwords are in use */
+#define MAX_PW_CRC	2  	/* max # of CRC-32's in password */
+#define PW_FILE_SUFFIX ".crc"  	/* suffix for the file that saves password CRC's */
+#endif
+
+#ifndef MAX_SECONDARY_FILESIZE
+#define MAX_SECONDARY_FILESIZE 10240
+#endif
+
+#define MAX_SECONDARY ((MAX_SECONDARY_FILESIZE+SECTOR_SIZE-1)/SECTOR_SIZE)
+
+#ifdef LCF_DSECS
+#define MAX_DESCR_SECS LCF_DSECS
+#else
+#define MAX_DESCR_SECS 2	/* maximum # of descriptor sectors (8 max) */
+#endif
+#define MAX_IMAGES      ((SECTOR_SIZE*MAX_DESCR_SECS-sizeof(long)-1)/sizeof(IMAGE_DESCR))
+			  /* maximum number of images */
 
 #define SECTOR_SIZE	512 /* disk sector size */
 #ifndef BLOCK_SIZE /* may also come from linux/fs.h */
 #define BLOCK_SIZE	1024 /* disk block size */
 #endif
 
-#define ADDR_OFFS MAX_IMAGE_NAME+MAX_PW+2 /* options address offset */
+#if (MAX_SECONDARY_FILESIZE > MAX_SECONDARY*SECTOR_SIZE)
+#error "Largest secondary loader exceeds sector addresses reserved."
+#endif
 
 #define PARTITION_ENTRY	   16	/* size of one partition entry */
 #define PARTITION_ENTRIES  4    /* number of partition entries */
@@ -74,17 +119,17 @@
 #define PART_ACT_ENT_OFF   0	/* offset in entry for active flag */
 #define PART_TYPE_ENT_OFF  4	/* offset in entry for partition type */
 
-#define PART_MASK  15 /* largest common partition bits mask */
+#define P_MASK(x) ((~has_partitions(x))&0xFFFF)
+#define D_MASK(x) (has_partitions(x))
 #define PART_MAX    4 /* biggest primary partition number */
 
 #define MAX_BOOT_SIZE	0x1b6	/* (leave some space for NT's and DR DOS' dirty
 				   hacks) scream if the boot sector gets any
 				   bigger */
 
-#define INIT_CKS	0xabcd	/* initial descriptor table checksum */
-
 #define BOOT_SIGNATURE	0xaa55	/* boot signature */
 #define BOOT_SIG_OFFSET	510	/* boot signature offset */
+#define MAGIC_SERIAL	0xC9CF	/* LILO installed serial number */
 
 #define PART_LINUX_MINIX  0x81	/* Linux/MINIX partition */
 #define PART_LINUX_SWAP	  0x82	/* Linux swap partition */
@@ -112,25 +157,36 @@
 #define STAGE_FIRST	1	/* first stage loader code */
 #define STAGE_SECOND	2	/* second stage loader code */
 #define	STAGE_CHAIN	0x10	/* chain loader code */
+#define STAGE_DRIVE_MAP	0x11	/* chain loader drive mapper */
+#define STAGE_FLAG_SERIAL  0x0100	/* boot loader supports serial i/o */
+#define STAGE_FLAG_MENU	0x0200	/* boot loader supports menu interface */
+#define STAGE_FLAG_BMP4	0x0400	/* boot loader supports 640x480x4 bitmaps */
 
 #define SETUPSECS	4	/* nr of setup sectors */
 #define VSS_NUM		497	/* address where variable setup size is
 				   stored */
-#ifndef LCF_LARGE_EBDA
+#define MAX_SETUPSECS	31	/* maximum number of sectors in kernel
+				   setup code (+ bootsect) */
 #define MAX_KERNEL_SECS	1024	/* absolute maximum kernel size */
-#else
-#define MAX_KERNEL_SECS	(1024-MAX_SECONDARY-5-1)
-			/* abs. max. kernel size (less 9k is gone) */
-#endif
 #define SPECIAL_SECTORS	2	/* special sectors (don't compact) at beginning
 				   of map sections */
 
 #define LINEAR_FLAG	0x40	/* mark linear address */
 #define LBA32_FLAG      0x20    /* mark lba 32-bit address */
-#define LBA32_NOCOUNT   0x10    /* mark address with count absent */
+#define LBA32_NOCOUNT   0x40    /* mark address with count absent */
+#define RAID_REL_FLAG	0x10	/* mark address raid-relocatable */
+/*
+ *	FLAG	Description
+ *
+ *   	0x00	 pure geometric addressing (C:H:S)
+ *	0x40	 Linear address (24-bits) converted to CHS at boot-time
+ *	0x60	 LBA32 address (32-bits), count=1, sets the high nibble!!
+ *	0x20	 LBA32 address (24-bits) + (8-bit) high nibble (implied)
+ *
+*/
 #define DEV_MASK        0x8f    /* mask to actual device code */
 
-#define	EX_OFF		CODE_START_1+SSDIFF /* external parameter block */
+#define	EX_OFF		SETUP_STACKSIZE-8+SSDIFF /* external parameter block */
 #define EX_DL_MAG	0xfe	/* magic number in DL */
 #define EX_MAG_L	0x494c	/* magic number at ES:SI, "LI" */
 #define EX_MAG_H	0x4f4c	/* magic number at ES:SI+2, "LO" */
@@ -141,6 +197,15 @@
 #define BIOS_MAX_CYLS   1024	/* 10 bits cylinder number */
 #define BIOS_MAX_SECS   63	/* 6 bits sector number (no, it's not 64) */
 
+/* these are the boot record flags in the "prompt" variable */
+#define FLAG_PROMPT	1	/* always issue boot: prompt */
+#define FLAG_RAID	2	/* one boot record of many */
+#define FLAG_RAID_DEFEAT  4	/* defeat finding this RAID boot record */
+#define FLAG_RAID_NOWRITE 8	/* defeat RAID writeback of command line */
+#define FLAG_NOBD	16	/* defeat BIOS data collection at boot time */
+#define FLAG_SAVE (FLAG_PROMPT|FLAG_NOBD)  /* All but raid flags */
+
+/* these are the descriptor flags */
 #define FLAG_VGA	1	/* override VGA mode */
 #define FLAG_RESTR	2	/* restrict additional parameters */
 #define FLAG_LOCK	4	/* lock on target */
@@ -148,6 +213,7 @@
 #define FLAG_KERNEL	16	/* image is a kernel */
 #define FLAG_SINGLE	32	/* single key activation */
 #define FLAG_FALLBACK	64	/* fallback command line exists */
+#define FLAG_PASSWORD	128	/* this image requires a password */
 
 #define VGA_NOCOVR	0x8000	/* VGA setting not overridden on command line */
 
@@ -157,6 +223,7 @@
 #define DC_MGOFF	0x6b6d	/* magic number for disabled line */
 
 #define MAX_MESSAGE	65535	/* maximum message length */
+#define MAX_MENU_TITLE	37	/* maximum MENU title length */
 
 #define NEW_HDR_SIG	"HdrS"	/* setup header signature */
 #define NEW_HDR_VERSION	0x200	/* header version number */
@@ -168,9 +235,22 @@
 #define PRTMAP_SIZE	32	/* number of partition type mappings */
 #define DRVMAP_SIZE	8	/* number of drive mappings */
 
-#ifndef LILO_ASM
-#define BOOTSEG 0x07c0			/* for probe.c  */
-#else
+#define CRC_POLY1 0x04c11db7
+#define CRC_POLY2 0x23a55379
+#define CRC_POLY3 0x049f21c7
+#define CRC_POLY4 0x1c632927
+#define CRC_POLY5 0xA3139383
+
+#define PROBE_SIGNATURE	"LiLo"	/* signature placed in low memory */
+#define PROBESEG 0x60		/* must be in first 4k page in memory */
+#define EDD_LTH	    30		/* length of the EDD return structure (max) */
+#define EDD_PACKET  01		/* packet calls are supported */
+#define EDD_LOCK    02		/* removable media may be locked */
+#define EDD_SUBSET  04		/* EDD call supported */
+
+
+
+#ifdef LILO_ASM
 BOOTSEG   = 0x07C0			! original address of boot-sector
 PARTS_LOAD= 0x0600			! partition sector load address
 PARTS_SCR = 0x0800			! ditto, for non-boot partitions
@@ -180,36 +260,35 @@ INITSEG   = DEF_INITSEG			! we move boot here - out of the way
 SETUPSEG  = DEF_SETUPSEG		! setup starts here
 SYSSEG    = DEF_SYSSEG			! system loaded at 0x10000 (65536).
 
-MAPcode   = MAX_SECONDARY*0x200		! size of secondary code
+MAX_SECONDARY_asm1 = MAX_SECONDARY_FILESIZE + SECTOR_SIZE - 1
+MAX_SECONDARY_asm = MAX_SECONDARY_asm1 / SECTOR_SIZE	! define for the assembler
+MAX_DESCR_SECS_asm = MAX_DESCR_SECS	! **
+MAX_IMAGE_NAME_asm = MAX_IMAGE_NAME	! **
+MAX_PW_CRC_asm	= MAX_PW_CRC		! **
+SECTOR_SIZE_asm = SECTOR_SIZE		! **
+MAX_MENU_TITLE_asm = MAX_MENU_TITLE	! **
+
+MAPcode   = MAX_SECONDARY_asm*0x200	! size of secondary code
 MAP	  = MAPcode+0x200		! plus space for cmdline input
 DESCR	  = MAP+0x200	     ! 0x2200	! descriptor load area offset
-DFLCMD	  = DESCR+0x400	     ! 0x2600	! default command line
+DFLCMD	  = DESCR+0x200*MAX_DESCR_SECS_asm	     ! 0x2600	! default command line
 KEYTABLE  = DFLCMD+0x200     ! 0x2800	! keyboard translation table offset
 DATAEND	  = KEYTABLE+0x200   ! 0x2A00	! PARAMLINE or Load-Low area
 
 STACKSEG  = 0x9000	     ! MUST == INITSEG for kernel 2.0.36 (and others?)
 SETUP_STACKSIZE = 2048		! stacksize for kernel setup.S
 
+#else
+#define BOOTSEG 0x07c0			/* for probe.c  */
 #endif
+
 #define FIRSTSEG BOOTSEG
 
 #ifdef LILO_ASM
 
-#ifndef LCF_LARGE_EBDA
 STACK	  = 2048		! amount of stack space to reserve
 PARMLINE  = DATAEND		! 0x2A00	! parameter line construction area off.
 
-#else
-STKseg	  = FIRSTSEG-STACKSEG
-STACK	  = STKseg<<4	! 0xB000	! top of stack
-
-SECseg    = 0x10000-DATAEND		! min place for second segment
-SECONDSEG = SECseg>>4			! seg. register contents
-!SECONDSEG = 0x0D60	! 0x0B00	! second stage loader segment
-PARMLINE  = STACK+0x200	! 0x8000	! parameter line construction area
-					! direct offset into INITSEG
-#endif
-
 #if 0
 SSDIFFseg = FIRSTSEG-STACKSEG
 SSDIFF	  = SSDIFFseg<<4     ! 0xA000	! used to address FIRSTSEG from SS
@@ -221,13 +300,8 @@ SSDIFF	= 0
 				! as the max. heap pointer in newer kernels
 BOOTSECT  = 0x200		! kernel bootsect.S
 
-#ifndef LCF_LARGE_EBDA
 #define SETUP_STACK_DYN  PARMLINE
 #define SLA_SIZE_DYN  SETUP_STACK_DYN-SETUP_STACKSIZE-BOOTSECT
-#else
-#define SETUP_STACK  PARMLINE
-#define SLA_SIZE  SETUP_STACK-SETUP_STACKSIZE-BOOTSECT
-#endif
 
 KBBEG     = 0x41A			! beginning of keyboard buffer
 KBEND	  = 0x41C			! end of keyboard buffer
@@ -235,7 +309,7 @@ KBLOW	  = 0x1e
 KBHIGH	  = 0x3e
 
 !
-!  Memory layout without LARGE_EBDA option
+!  Memory layout
 !
 ! 0x007BE-0x007FD    64 B    partition table
 ! 0x07C00-0x07DFF   0.5 kB   HD/FD boot load address
@@ -282,3 +356,4 @@ CL_POINTER	= 0x228			! new pointer is dword address
 #endif
 
 #endif
+
