@@ -77,39 +77,68 @@ void map_patch_first(char *name,char *str)
 
 void map_create(char *name)
 {
-    char buffer[SECTOR_SIZE*4];
+    char buffer[SECTOR_SIZE*(MAX_DESCR_SECS+2)];
     int fd;
 
     if ((fd = creat(name,0600)) < 0) die("creat %s: %s",name,strerror(errno));
     (void) close(fd);
-    memset(buffer,0,SECTOR_SIZE*4);
+    memset(buffer,0,SECTOR_SIZE*(MAX_DESCR_SECS+2));
     *(unsigned short *) buffer = DC_MGOFF;
     map_file = geo_open(&map_geo,name,O_RDWR);
-    if (write(map_file,buffer,SECTOR_SIZE*4) != SECTOR_SIZE*4)
+    if (do_md_install) {
+	struct stat st;
+	if(fstat(map_file,&st)) die("map_create: cannot fstat map file");
+	if (verbose >= 2)
+	    printf("map_create:  boot=%04X  map=%04X\n", 
+					boot_dev_nr, (int)st.st_dev);
+	if (boot_dev_nr != st.st_dev) {
+#if 1
+	    die("map file must be on the boot RAID partition");
+#else
+	    raid_flags |= FLAG_RAID_NOWRITE;
+	    fprintf(errstd,
+	    	"Error: map file must be on the boot RAID partition\n"
+	    	"    This installation provides no redundancy\n"	);
+#endif
+	}
+    }
+    if (write(map_file,buffer,SECTOR_SIZE*(MAX_DESCR_SECS+2)) != SECTOR_SIZE*(MAX_DESCR_SECS+2))
 	die("write %s: %s",name,strerror(errno));
-    if (!geo_comp_addr(&map_geo,SECTOR_SIZE*3,&zero_addr))
+    if (!geo_comp_addr(&map_geo,SECTOR_SIZE*(MAX_DESCR_SECS+1),&zero_addr))
 	die("Hole found in map file (zero sector)");
 }
 
 
-void map_done(DESCR_SECTORS *descr,SECTOR_ADDR addr[3])
+void map_done(DESCR_SECTORS *descr,SECTOR_ADDR* addr)
 {
     struct stat st;
-    int i;
+    int i, pos;
+
+    descr->l.checksum =
+                crc32(descr->sector, sizeof((*descr).l.sector), CRC_POLY1);
 
-    descr->d.checksum = INIT_CKS;
-    for (i = 1; i < sizeof(DESCR_SECTORS)/sizeof(unsigned short); i++)
-	descr->d.checksum ^= ((unsigned short *) descr)[i];
     if (lseek(map_file,SECTOR_SIZE,0) < 0) pdie("lseek map file");
-    if (write(map_file,(char *) descr,SECTOR_SIZE*2) != SECTOR_SIZE*2)
+    if (write(map_file,(char *) descr,SECTOR_SIZE*MAX_DESCR_SECS) != SECTOR_SIZE*MAX_DESCR_SECS)
 	pdie("write map file");
+#if 0
     if (!geo_comp_addr(&map_geo,SECTOR_SIZE,&addr[0]))
 	die("Hole found in map file (first descr. sector)");
     if (!geo_comp_addr(&map_geo,SECTOR_SIZE*2,&addr[1]))
 	die("Hole found in map file (second descr. sector)");
     if (!geo_comp_addr(&map_geo,0,&addr[2]))
 	die("Hole found in map file (default command line)");
-    if (verbose > 1) {
+#else
+    pos = SECTOR_SIZE;
+    for (i=0; i<MAX_DESCR_SECS; i++) {
+	if (!geo_comp_addr(&map_geo,pos,addr))
+	    die("Hole found in map file (descr. sector %d)", i);
+	addr++;
+	pos += SECTOR_SIZE;
+    }
+    if (!geo_comp_addr(&map_geo,0,addr))
+	die("Hole found in map file (default command line)");
+#endif
+    if (verbose >= 2) {
 	if (fstat(map_file,&st) < 0) pdie("fstat map file");
 	printf("Map file size: %ld bytes.\n",(long) st.st_size);
     }
@@ -181,7 +210,10 @@ static void map_compact(int dont_compact)
     hinib = 257;
     maxcount = lba32 ? 127 : 128;  /* JRC: max LBA transfer is 127 sectors,
 	      per the EDD spec, v1.1, not 128 (unfortunately) */
-/* JRC: for testing the hinib save:   maxcount = lba32 ? 3 : maxcount; */
+/* JRC: for testing the hinib save: */
+#ifdef DEBUG
+	maxcount = lba32 ? 3 : maxcount;
+#endif
     walk = map;
     for (count = 0; walk && count <= dont_compact; count++) walk = walk->next;
     offset = 0;
@@ -262,7 +294,7 @@ int map_end_section(SECTOR_ADDR *addr,int dont_compact)
     for (walk = map; walk; walk = next) {
 	next = walk->next;
 	if (verbose > 3) {
-	    if (walk->addr.device&LBA32_NOCOUNT) hinib = walk->addr.num_sect;
+	    if ((walk->addr.device&LBA32_FLAG) && (walk->addr.device&LBA32_NOCOUNT)) hinib = walk->addr.num_sect;
 	    printf("  Mapped AL=0x%02x CX=0x%04x DX=0x%04x",walk->addr.num_sect,
 	      (walk->addr.track << 8) | walk->addr.sector,(walk->addr.head << 8)
 	      | walk->addr.device);
@@ -285,7 +317,8 @@ int map_end_section(SECTOR_ADDR *addr,int dont_compact)
 	}
 	memcpy(buffer+offset,&walk->addr,sizeof(SECTOR_ADDR));
 	offset += sizeof(SECTOR_ADDR);
-	sectors += walk->addr.device&LBA32_NOCOUNT ? 1 : walk->addr.num_sect;
+	sectors += (walk->addr.device&LBA32_FLAG) && (walk->addr.device&LBA32_NOCOUNT)
+		? 1 : walk->addr.num_sect;
 	free(walk);
     }
     if (offset)
