@@ -10,15 +10,19 @@ source directory.
 */
 
 
-#include <unistd.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <stdio.h>
+#include <string.h>
+#include <ctype.h>
 #include <fcntl.h>
-#include <limits.h>
 #include <errno.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <linux/unistd.h>
+#include <limits.h>
 #include <linux/fs.h>
-#include <string.h>
+#include <time.h>
 #include "config.h"
 #include "lilo.h"
 #include "common.h"
@@ -29,15 +33,19 @@ source directory.
 #include "boot.h"
 
 
-/* For older kernels ... */
- 
-#ifndef DOS_EXTENDED_PARTITION
-#define DOS_EXTENDED_PARTITION EXTENDED_PARTITION
-#endif
+       
+       _syscall5(int,  _llseek,  uint,  fd, ulong, hi, ulong, lo,
+       loff_t *, res, uint, wh);
 
-#ifndef LINUX_EXTENDED_PARTITION
-#define LINUX_EXTENDED_PARTITION EXTENDED_PARTITION
-#endif
+       int _llseek(unsigned int fd,  unsigned  long  offset_high,
+       unsigned  long  offset_low,  loff_t * result, unsigned int
+       whence);
+       
+       loff_t llseek(unsigned int fd, loff_t offs, unsigned int whence)
+       { loff_t res;
+       	return _llseek(fd, offs>>32, offs, &res, whence) < 0  ?
+       			 (loff_t)(-1) : res;
+       }
 
 
 void part_verify(int dev_nr,int type)
@@ -45,52 +53,75 @@ void part_verify(int dev_nr,int type)
     GEOMETRY geo;
     DEVICE dev;
     char backup_file[PATH_MAX+1];
-    int fd,bck_file,part,size,lin_3d,cyl;
+    int fd, bck_file, part, size, lin_3d, cyl;
+    unsigned long second, base;
     struct partition part_table[PART_MAX];
-#if 1
-    int mask;
+    int mask, i, pe;
+    unsigned short boot_sig;
     
-    if (!(mask = has_partitions(dev_nr)) || !(dev_nr & mask) ||
-        (dev_nr & mask) > PART_MAX ) return;
-#else
-    if (!(dev_nr & PART_MASK) || (dev_nr & PART_MASK) > PART_MAX ||
-      (MAJOR(dev_nr) != MAJOR_HD && MAJOR(dev_nr) != MAJOR_XT &&
-       MAJOR(dev_nr) != MAJOR_SD && MAJOR(dev_nr) != MAJOR_IDE2 &&
-       MAJOR(dev_nr) != MAJOR_IDE3 && MAJOR(dev_nr) != MAJOR_IDE4 &&
-       MAJOR(dev_nr) != MAJOR_IDE5 && MAJOR(dev_nr) != MAJOR_IDE6 &&
-       MAJOR(dev_nr) != MAJOR_I2O &&
-      MAJOR(dev_nr) != MAJOR_DAC960)) return;
-#endif      
-    geo_get(&geo,dev_nr & ~PART_MASK,-1,1);
-    fd = dev_open(&dev,dev_nr & ~PART_MASK,cfg_get_flag(cf_options,"fix-table")
+    if (!has_partitions(dev_nr) || !(mask = P_MASK(dev_nr)) || !(dev_nr & mask)
+#if 0
+     || (dev_nr & mask) > PART_MAX
+#endif
+     	) return;
+
+    if (verbose >= 4) printf("part_verify:  dev_nr=%04x, type=%d\n", dev_nr, type);
+    geo_get(&geo,dev_nr & ~mask,-1,1);
+    fd = dev_open(&dev,dev_nr & ~mask,cfg_get_flag(cf_options,"fix-table")
       && !test ? O_RDWR : O_RDONLY);
-    part = (dev_nr & PART_MASK)-1;
-    if (lseek(fd,PART_TABLE_OFFSET,0) < 0) pdie("lseek partition table");
-    if (!(size = read(fd,(char *) part_table,sizeof(struct partition)*
+    part = (pe = dev_nr & mask)-1;
+    if (lseek(fd, PART_TABLE_OFFSET, SEEK_SET) < 0) pdie("lseek partition table");
+    if (!(size = read(fd,(char *) part_table, sizeof(struct partition)*
       PART_MAX))) die("Short read on partition table");
     if (size < 0) pdie("read partition table");
+    if ( read(fd, &boot_sig, sizeof(boot_sig)) != sizeof(boot_sig)  ||
+	boot_sig != BOOT_SIGNATURE ) die("read boot signature failed");
+
+    if (verbose>=5) printf("part_verify:  part#=%d\n", pe);
+
+    second=base=0;
+    for (i=0; i<PART_MAX; i++) {
+	if (is_extd_part(part_table[i].sys_ind)) {
+	    if (!base) base = part_table[i].start_sect;
+	    else die("invalid partition table: second extended partition found");
+	}
+    }
+    i=5;
+    while (i<=pe && base) {
+        if (llseek(fd, SECTORSIZE*(base+second) + PART_TABLE_OFFSET, SEEK_SET) < 0)
+            die("secondary llseek failed");
+	if (read(fd, part_table, sizeof(part_table)) != sizeof(part_table)) die("secondary read pt failed");
+	if ( read(fd, &boot_sig, sizeof(boot_sig)) != sizeof(boot_sig)  ||
+	    boot_sig != BOOT_SIGNATURE ) die("read second boot signature failed");
+        if (is_extd_part(part_table[1].sys_ind)) second=part_table[1].start_sect;
+        else base = 0;
+        i++;
+        part=0;
+    }
+
     if (type && part_table[part].sys_ind != PART_LINUX_MINIX &&
-      part_table[part].sys_ind != PART_LINUX_NATIVE &&
-      part_table[part].sys_ind != DOS_EXTENDED_PARTITION &&
-      part_table[part].sys_ind != LINUX_EXTENDED_PARTITION) {
+	      part_table[part].sys_ind != PART_LINUX_NATIVE &&
+	      !is_extd_part(part_table[part].sys_ind) ) {
 	fflush(stdout);
-	fprintf(stderr,"Device 0x%04X: Partition type 0x%02X does not seem "
-	  "suitable for a LILO boot sector\n",dev_nr,part_table[part].sys_ind);
+	fprintf(errstd,"Device 0x%04X: Partition type 0x%02X does not seem "
+	    "suitable for a LILO boot sector\n",dev_nr,part_table[part].sys_ind);
 	if (!cfg_get_flag(cf_options,"ignore-table")) exit(1);
     }
     cyl = part_table[part].cyl+((part_table[part].sector >> 6) << 8);
     lin_3d = (part_table[part].sector & 63)-1+(part_table[part].head+
       cyl*geo.heads)*geo.sectors;
-    if ((lin_3d > part_table[part].start_sect || (lin_3d <
-      part_table[part].start_sect && cyl != BIOS_MAX_CYLS-1)) && !nowarn) {
+    if (pe <= PART_MAX &&
+	    (lin_3d > part_table[part].start_sect || (lin_3d <
+	    part_table[part].start_sect && cyl != BIOS_MAX_CYLS-1)) &&
+	    !nowarn) {
 	fflush(stdout);
-	fprintf(stderr,"Device 0x%04X: Invalid partition table, %d%s entry\n",
-	  dev_nr & ~PART_MASK,part+1,!part ? "st" : part == 1 ? "nd" : part ==
+	fprintf(errstd,"Device 0x%04X: Invalid partition table, %d%s entry\n",
+	  dev_nr & ~mask,part+1,!part ? "st" : part == 1 ? "nd" : part ==
 	  2 ? "rd" : "th");
-	fprintf(stderr,"  3D address:     %d/%d/%d (%d)\n",part_table[part].
+	fprintf(errstd,"  3D address:     %d/%d/%d (%d)\n",part_table[part].
 	  sector & 63,part_table[part].head,cyl,lin_3d);
 	cyl = part_table[part].start_sect/geo.sectors/geo.heads;
-	fprintf(stderr,"  Linear address: %d/%d/%d (%d)\n",part_table[part].
+	fprintf(errstd,"  Linear address: %d/%d/%d (%d)\n",part_table[part].
 	  sector = (part_table[part].start_sect % geo.sectors)+1,part_table
 	  [part].head = (part_table[part].start_sect/geo.sectors) %
 	  geo.heads,cyl,part_table[part].start_sect);
@@ -99,9 +130,9 @@ void part_verify(int dev_nr,int type)
 	if (!cfg_get_flag(cf_options,"fix-table") && !cfg_get_flag(cf_options,
 	  "ignore-table")) exit(1);
 	if (test || cfg_get_flag(cf_options,"ignore-table"))
-	    fprintf(stderr,"The partition table is *NOT* being adjusted.\n");
+	    fprintf(errstd,"The partition table is *NOT* being adjusted.\n");
 	else {
-	    sprintf(backup_file,BACKUP_DIR "/part.%04X",dev_nr & ~PART_MASK);
+	    sprintf(backup_file,BACKUP_DIR "/part.%04X",dev_nr & ~mask);
 	    if ((bck_file = creat(backup_file,0644)) < 0)
 		die("creat %s: %s",backup_file,strerror(errno));
 	    if (!(size = write(bck_file,(char *) part_table,
@@ -113,7 +144,7 @@ void part_verify(int dev_nr,int type)
 	    if (verbose > 0)
 		printf("Backup copy of partition table in %s\n",backup_file);
 	    printf("Writing modified partition table to device 0x%04X\n",
-	      dev_nr & ~PART_MASK);
+	      dev_nr & ~mask);
 	    if (lseek(fd,PART_TABLE_OFFSET,0) < 0)
 		pdie("lseek partition table");
 	    if (!(size = write(fd,(char *) part_table,sizeof(struct partition)*
@@ -264,7 +295,7 @@ void do_cr_auto(void)
     if (i>PART_MAX || i<1) return;
 #endif
     table = cfg_get_strg(cf_other,"table");
-    table2 = boot_table(other);	/* get possible default */
+    table2 = boot_mbr(other, 1);	/* get possible default */
     if (!table) table = table2;
     
     if (!table && autoauto) return;
@@ -275,10 +306,9 @@ void do_cr_auto(void)
     /*    
      */
     if (stat(table,&st) < 0) die("stat %s: %s",table,strerror(errno));
-    geo_get(&geo,st.st_rdev & ~PART_MASK,-1,1);
-    partition = st.st_rdev & PART_MASK;
+    geo_get(&geo,st.st_rdev & D_MASK(st.st_rdev),-1,1);
+    partition = st.st_rdev & P_MASK(st.st_rdev);
     if (!S_ISBLK(st.st_mode) || partition)
-/***	cfg_error("\"%s\" isn't a primary partition",table);	***/
 	cfg_error("\"%s\" doesn't contain a primary partition table",table);
     pfd = open(table, O_RDONLY);
     if (pfd<0) die("Cannot open %s", table);
@@ -300,7 +330,7 @@ void do_cr_auto(void)
 	    j = i*PARTITION_ENTRY + PART_TYPE_ENT_OFF;
 	    if (autoauto && !nowarn) {
 	    	fflush(stdout);
-	        fprintf(stderr, "Warning: CHANGE AUTOMATIC assumed after \"other=%s\"\n", other);
+	        fprintf(errstd, "Warning: CHANGE AUTOMATIC assumed after \"other=%s\"\n", other);
 	        autoauto = 0;  /* suppress further warnings */
 	    }    
 	    if (i == partition-1)
@@ -310,7 +340,7 @@ void do_cr_auto(void)
 	}
     }
 #else
-    if (!nowarn) fprintf(stderr,
+    if (!nowarn) fprintf(errstd,
        "Warning:  This LILO is compiled without REWRITE_TABLE;\n"
        "   unable to generate CHANGE/AUTOMATIC change-rules\n");
 #endif
@@ -327,8 +357,8 @@ void do_cr_part(void)
 
     tmp = cfg_get_strg(cf_change,"partition");
     if (stat(tmp,&st) < 0) die("stat %s: %s",tmp,strerror(errno));
-    geo_get(&geo,st.st_rdev & ~PART_MASK,-1,1);
-    partition = st.st_rdev & PART_MASK;
+    geo_get(&geo,st.st_rdev & D_MASK(st.st_rdev),-1,1);
+    partition = st.st_rdev & P_MASK(st.st_rdev);
     if (!S_ISBLK(st.st_mode) || !partition || partition > PART_MAX)
 	cfg_error("\"%s\" isn't a primary partition",tmp);
     part_base = (partition-1)*PARTITION_ENTRY;
@@ -404,3 +434,105 @@ void preload_types(void)
     add_type("DOS16_small", PART_DOS16_SMALL, -1);
     add_type("DOS12", PART_DOS12, -1);
 }
+
+
+
+#define PART_BEGIN	0x1be
+#define PART_NUM	4
+#define PART_SIZE	16
+#define PART_ACTIVE	0x80
+#define PART_INACTIVE	0
+
+
+void do_activate(char *where, char *which)
+{
+    struct stat st;
+    int fd,number,count;
+    unsigned char flag, ptype;
+
+    if ((fd = open(where, !which ? O_RDONLY : O_RDWR)) < 0)
+	die("open %s: %s",where,strerror(errno));
+    if (fstat(fd,&st) < 0) die("stat %s: %s",where,strerror(errno));
+    if (!S_ISBLK(st.st_mode)) die("%s: not a block device",where);
+    if (verbose >= 1) {
+       printf("st.st_dev = %04X, st.st_rdev = %04X\n",
+       				(int)st.st_dev, (int)st.st_rdev);
+    }
+    if ((st.st_rdev & has_partitions(st.st_rdev)) != st.st_rdev)
+        die("%s is not a master device with a primary partition table", where);
+    if (!which) {	/* one argument: display active partition */
+	for (count = 1; count <= PART_NUM; count++) {
+	    if (lseek(fd,PART_BEGIN+(count-1)*PART_SIZE,0) < 0)
+		die("lseek: %s",strerror(errno));
+	    if (read(fd,&flag,1) != 1) die("read: %s",strerror(errno));
+	    if (flag == PART_ACTIVE) {
+		printf("%s%d\n",where,count);
+		exit(0);
+	    }
+	}
+	die("No active partition found on %s",where);
+    }
+    number = to_number(which);
+    if (number < 1 || number > 4)
+	die("%s: not a valid partition number (1-4)",which);
+    for (count = 1; count <= PART_NUM; count++) {
+	if (lseek(fd,PART_BEGIN+(count-1)*PART_SIZE+4,0) < 0)
+	    die("lseek: %s",strerror(errno));
+	if (read(fd,&ptype,1) != 1) die("read: %s",strerror(errno));
+	if (count == number && ptype==0) die("Cannot activate an empty partition");
+    }
+    if (test) {
+        printf("The partition table of  %s  has *NOT* been updated\n",where);
+    }
+    else for (count = 1; count <= PART_NUM; count++) {
+	if (lseek(fd,PART_BEGIN+(count-1)*PART_SIZE,0) < 0)
+	    die("lseek: %s",strerror(errno));
+	flag = count == number ? PART_ACTIVE : PART_INACTIVE;
+	if (write(fd,&flag,1) != 1) die("write: %s",strerror(errno));
+    }
+    exit(0);
+}
+
+
+void do_install_mbr(char *where, char *what)
+{
+    int fd, nfd, i;
+    struct stat st;
+    char buf[SECTOR_SIZE];
+    
+    if (!what) what = DFL_MBR;
+    if ((fd=open(where,O_RDWR)) < 0) die("Cannot open %s: %s", where,strerror(errno));
+    if (fstat(fd,&st) < 0) die("stat: %s : %s", where,strerror(errno));
+    if (!S_ISBLK(st.st_mode)) die("%s not a block device",where);
+    if (st.st_rdev != (st.st_rdev & has_partitions(st.st_rdev)))
+	die("%s is not a master device with a primary parition table",where);
+    if (read(fd,buf,SECTOR_SIZE) != SECTOR_SIZE) die("read %s: %s",where, strerror(errno));
+    
+    if ((nfd=open(what,O_RDONLY)) < 0) die("Cannot open %s: %s",what,strerror(errno));
+    if (read(nfd,buf,MAX_BOOT_SIZE) != MAX_BOOT_SIZE) die("read %s: %s",what,strerror(errno));
+    
+    *(unsigned short*)&buf[BOOT_SIG_OFFSET] = BOOT_SIGNATURE;
+    if (zflag) {
+        char *p = buf+MAX_BOOT_SIZE;
+        for (i=0; i<8; i++) *p++ = 0;
+    } else if (*(int*)&buf[PART_TABLE_OFFSET - 6] == 0) {
+    	i = st.st_rdev;
+    	i %= 271;		/* modulo a prime number; eg, 2551, 9091 */
+        srand(time(NULL));	/* seed the random number generator */
+        while (i--) rand();
+        *(int*)&buf[PART_TABLE_OFFSET - 6] = rand();  /* insert serial number */
+        if (*(short*)&buf[PART_TABLE_OFFSET - 2] == 0)
+            *(short*)&buf[PART_TABLE_OFFSET - 2] = MAGIC_SERIAL;
+    }
+    
+    if (lseek(fd,0,SEEK_SET) != 0) die("seek %s; %s", where, strerror(errno));
+    if (!test) {
+	if (write(fd,buf,SECTOR_SIZE) != SECTOR_SIZE)
+		die("write %s: %s",where,strerror(errno));
+    }
+    close(fd); close(nfd);
+    printf("The Master Boot Record of  %s  has %sbeen updated.\n", where, test ? "*NOT* " : "");
+    sync();
+    exit(0);
+}
+
