@@ -6,6 +6,9 @@
 #ifndef PARTITION_H
 #define PARTITION_H
 
+#include <unistd.h>
+#include <linux/unistd.h>
+
 typedef struct _change_rule {
     const char *type;
     unsigned char normal;
@@ -13,6 +16,15 @@ typedef struct _change_rule {
     struct _change_rule *next;
 } CHANGE_RULE;
 
+#if 1
+
+#define SECTORSIZE ((long long)SECTOR_SIZE)
+
+       loff_t llseek(unsigned int fd, loff_t offs, unsigned int whence);
+
+#endif
+
+#define is_extd_part(x) ((x)==PART_DOS_EXTD||(x)==PART_WIN_EXTD_LBA||(x)==PART_LINUX_EXTD)
 
 void part_verify(int dev_nr,int type);
 /* Verify the partition table of the disk of which dev_nr is a partition. May
@@ -25,4 +37,11 @@ void do_cr_auto(void);
 void preload_types(void);
 /* Preload some partition types for convenience */
 
+void do_activate(char *where, char *which);
+/* Activate the specified partition */
+
+void do_install_mbr(char *where, char *what);
+/* Install a new MBR (Master Boot Record) */
+
+
 #endif
